
# q-expansion of newform 1920.2.b.d, downloaded from the LMFDB on 27 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1920
weight = 2
poly_data = [7, 0, -16, 8, 14, -8, 8, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[-12187, 5516, 29943, -1256, 13117, 5184, 1999, 510], 8561], [[18046, -5306, -6072, 24917, -10690, 6815, -703, 720], 8561], [[2743, -3540, -6669, 540, -1951, -998, -357, -118], 1223], [[-3031, -42392, 23587, -2509, -5867, 8229, 592, 1196], 8561], [[2142, 39844, -14076, -19676, 13354, -11052, 316, -1444], 8561], [[6055, 11018, 3015, -25125, 13353, -12609, -2, -1508], 8561], [[11991, 798, -9087, 50042, -24043, 19424, -701, 2228], 8561]]

hecke_ring_character_values = [[511, [-1, 0, 0, 0, 0, 0, 0, 0]], [901, [-1, 0, 0, 0, 0, 0, 0, 0]], [641, [-1, 0, 0, 0, 0, 0, 0, 0]], [1537, [1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 1, 0], [-1, 0, 0, 0, 0, 0, 0, 0], [0, -1, 0, 0, 0, 0, 0, 0], [0, 2, 0, 0, 0, 0, 0, 0], [0, 0, 0, -1, 0, -1, 0, 1], [0, 0, 0, -1, 0, 0, 0, -1], [0, 0, 0, 0, -1, 0, 1, 0], [0, 0, 1, 0, 1, 0, 0, 0], [2, 0, 0, 0, 0, 1, 0, 0], [0, 1, -1, 0, 0, 0, 1, 0], [0, 0, 0, -1, 0, 1, 0, -3], [0, 0, 0, -2, 0, -1, 0, 0], [0, 0, -1, 0, -2, 0, 1, 0], [0, 0, -3, 0, -1, 0, -2, 0], [-2, 0, 0, 0, 0, 0, 0, 0], [0, 0, -4, 0, 0, 0, 4, 0], [0, 0, 0, 4, 0, 1, 0, 2], [0, 0, 1, 0, -2, 0, 3, 0], [0, 0, 2, 0, -2, 0, 4, 0], [-2, 0, 0, 0, 0, -2, 0, 0], [0, -1, -5, 0, 0, 0, 5, 0], [0, 2, -3, 0, 0, 0, 3, 0], [0, 0, 0, 2, 0, 2, 0, -2], [-2, 0, 0, 0, 0, -4, 0, 0], [-10, 0, 0, 0, 0, -3, 0, 0], [0, 1, 8, 0, 0, 0, -8, 0], [0, 6, -1, 0, 0, 0, 1, 0], [0, 0, 0, -2, 0, -1, 0, 0], [0, 0, 0, 3, 0, 0, 0, 3], [0, 3, -2, 0, 0, 0, 2, 0], [0, -4, -6, 0, 0, 0, 6, 0], [0, 0, 0, 1, 0, -2, 0, 5], [0, 0, 0, 0, -1, 0, 1, 0], [-2, 0, 0, 0, 0, 4, 0, 0], [0, 7, 1, 0, 0, 0, -1, 0], [0, 0, 0, -5, 0, -1, 0, -3], [0, 0, 1, 0, 0, 0, 1, 0], [0, 0, 5, 0, -1, 0, 6, 0], [10, 0, 0, 0, 0, -4, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -4, 0, 8], [0, 0, -2, 0, -4, 0, 2, 0], [-2, 0, 0, 0, 0, -4, 0, 0], [-2, 0, 0, 0, 0, 0, 0, 0], [0, -7, 1, 0, 0, 0, -1, 0], [0, 0, -6, 0, -1, 0, -5, 0], [0, -9, 6, 0, 0, 0, -6, 0], [0, 4, 9, 0, 0, 0, -9, 0], [0, 0, 0, -2, 0, 0, 0, -2], [0, 0, 0, 1, 0, 4, 0, -7], [0, 0, -8, 0, -2, 0, -6, 0], [6, 0, 0, 0, 0, -5, 0, 0], [0, 2, 8, 0, 0, 0, -8, 0], [0, 0, 0, 1, 0, 4, 0, -7], [0, 0, 3, 0, 7, 0, -4, 0], [14, 0, 0, 0, 0, 4, 0, 0], [0, 3, -11, 0, 0, 0, 11, 0], [0, 0, 0, -1, 0, -1, 0, 1], [0, 0, 0, 2, 0, 3, 0, -4], [0, 0, 3, 0, 4, 0, -1, 0], [-6, 0, 0, 0, 0, -4, 0, 0], [0, 0, 7, 0, 4, 0, 3, 0], [0, 0, -2, 0, -6, 0, 4, 0], [18, 0, 0, 0, 0, -4, 0, 0], [6, 0, 0, 0, 0, -4, 0, 0], [0, 0, 6, 0, 5, 0, 1, 0], [2, 0, 0, 0, 0, -6, 0, 0], [0, -8, -3, 0, 0, 0, 3, 0], [0, 0, 0, 2, 0, 2, 0, -2], [0, 0, 0, -5, 0, -2, 0, -1], [0, 0, 4, 0, -2, 0, 6, 0], [0, -5, -12, 0, 0, 0, 12, 0], [0, 0, 0, -3, 0, -5, 0, 7], [0, 0, -8, 0, 1, 0, -9, 0], [0, 0, 5, 0, 7, 0, -2, 0], [-14, 0, 0, 0, 0, 4, 0, 0], [0, 0, 0, 5, 0, -1, 0, 7], [0, 0, 0, 0, 0, 2, 0, -4], [-2, 0, 0, 0, 0, -5, 0, 0], [0, -10, 2, 0, 0, 0, -2, 0], [0, 0, 0, 0, 0, 3, 0, -6], [0, 0, -6, 0, -6, 0, 0, 0], [-10, 0, 0, 0, 0, 4, 0, 0], [0, 1, -3, 0, 0, 0, 3, 0], [0, -12, 1, 0, 0, 0, -1, 0], [0, 0, 0, 8, 0, 1, 0, 6], [2, 0, 0, 0, 0, -2, 0, 0], [34, 0, 0, 0, 0, -1, 0, 0], [0, -11, 2, 0, 0, 0, -2, 0], [0, -14, 5, 0, 0, 0, -5, 0], [0, 0, 12, 0, 2, 0, 10, 0], [0, -5, -2, 0, 0, 0, 2, 0], [0, -4, 6, 0, 0, 0, -6, 0], [0, 0, 8, 0, 5, 0, 3, 0], [0, 0, -5, 0, 5, 0, -10, 0], [-2, 0, 0, 0, 0, 11, 0, 0], [0, 0, 0, 8, 0, 0, 0, 8], [0, 0, -7, 0, 2, 0, -9, 0], [0, 0, 0, -8, 0, -2, 0, -4], [0, 0, 5, 0, 0, 0, 5, 0], [10, 0, 0, 0, 0, -2, 0, 0], [0, 6, 7, 0, 0, 0, -7, 0], [0, 0, 0, 0, 0, 3, 0, -6], [0, 0, 14, 0, 1, 0, 13, 0], [-14, 0, 0, 0, 0, 4, 0, 0], [0, 4, -7, 0, 0, 0, 7, 0], [0, 0, 0, -1, 0, -2, 0, 3], [0, 0, -8, 0, 2, 0, -10, 0], [-14, 0, 0, 0, 0, -5, 0, 0], [0, 5, 14, 0, 0, 0, -14, 0], [0, 0, 0, -1, 0, 3, 0, -7], [0, 0, 0, 7, 0, 0, 0, 7], [0, 0, 4, 0, 3, 0, 1, 0], [0, -3, -13, 0, 0, 0, 13, 0], [0, 0, 0, 2, 0, -3, 0, 8], [0, 0, 1, 0, -2, 0, 3, 0], [0, 0, -7, 0, -3, 0, -4, 0], [22, 0, 0, 0, 0, -2, 0, 0], [0, 12, 8, 0, 0, 0, -8, 0], [0, 0, 0, -4, 0, 5, 0, -14], [-26, 0, 0, 0, 0, -4, 0, 0], [-26, 0, 0, 0, 0, -6, 0, 0], [0, 8, -5, 0, 0, 0, 5, 0], [0, 0, -10, 0, -3, 0, -7, 0], [10, 0, 0, 0, 0, 4, 0, 0], [0, 0, 0, -6, 0, -10, 0, 14], [0, 0, 4, 0, 0, 0, 4, 0], [0, -9, -18, 0, 0, 0, 18, 0], [0, 0, 0, 13, 0, 3, 0, 7], [0, 0, -4, 0, -7, 0, 3, 0], [0, 0, 1, 0, 3, 0, -2, 0], [0, 1, -9, 0, 0, 0, 9, 0], [0, 0, 0, -1, 0, -1, 0, 1], [0, 0, 0, -4, 0, 2, 0, -8], [18, 0, 0, 0, 0, 2, 0, 0], [6, 0, 0, 0, 0, -8, 0, 0], [0, 0, 7, 0, 0, 0, 7, 0], [-26, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 6, 0, 4, 0, -2], [0, 0, 6, 0, -5, 0, 11, 0], [-18, 0, 0, 0, 0, 0, 0, 0], [0, 1, -4, 0, 0, 0, 4, 0], [0, 2, -19, 0, 0, 0, 19, 0], [0, 0, 0, 2, 0, -3, 0, 8], [0, 0, -12, 0, -6, 0, -6, 0], [0, 0, 0, -1, 0, 3, 0, -7], [0, 0, 0, 9, 0, 10, 0, -11], [0, 0, -12, 0, -3, 0, -9, 0], [0, 0, 11, 0, 1, 0, 10, 0], [0, 0, 0, 7, 0, 7, 0, -7], [0, 0, 0, -6, 0, 3, 0, -12], [0, 0, -3, 0, -8, 0, 5, 0], [0, 0, -3, 0, 1, 0, -4, 0], [0, 0, -11, 0, -6, 0, -5, 0], [0, 0, -2, 0, -2, 0, 0, 0], [0, 3, -5, 0, 0, 0, 5, 0], [0, 0, 0, -12, 0, -1, 0, -10], [6, 0, 0, 0, 0, 18, 0, 0], [6, 0, 0, 0, 0, -11, 0, 0], [0, -12, -1, 0, 0, 0, 1, 0], [0, 0, 0, 7, 0, -4, 0, 15], [0, 13, 0, 0, 0, 0, 0, 0], [0, 2, 0, 0, 0, 0, 0, 0], [0, 0, 0, -5, 0, -6, 0, 7], [0, 0, -11, 0, -1, 0, -10, 0], [0, -17, 13, 0, 0, 0, -13, 0], [0, 0, 0, 1, 0, -3, 0, 7], [-6, 0, 0, 0, 0, 14, 0, 0], [-2, 0, 0, 0, 0, -8, 0, 0], [0, 0, -20, 0, 0, 0, 20, 0], [0, 0, 0, 0, 0, 2, 0, -4], [0, 0, -6, 0, 0, 0, -6, 0], [22, 0, 0, 0, 0, -4, 0, 0], [0, -17, -5, 0, 0, 0, 5, 0], [0, 0, 0, -6, 0, -8, 0, 10], [0, 0, 6, 0, -7, 0, 13, 0], [18, 0, 0, 0, 0, -1, 0, 0], [0, -3, 2, 0, 0, 0, -2, 0], [0, 0, 0, 6, 0, 11, 0, -16], [0, -5, -6, 0, 0, 0, 6, 0], [0, 26, -4, 0, 0, 0, 4, 0], [0, 0, 0, -1, 0, -7, 0, 13], [0, 0, 0, -17, 0, -2, 0, -13], [0, 0, 5, 0, -3, 0, 8, 0], [-18, 0, 0, 0, 0, 7, 0, 0], [0, 0, 0, 3, 0, 1, 0, 1], [0, 0, 11, 0, 8, 0, 3, 0], [-10, 0, 0, 0, 0, 7, 0, 0], [0, 0, 6, 0, 8, 0, -2, 0], [-22, 0, 0, 0, 0, 14, 0, 0], [0, -2, -23, 0, 0, 0, 23, 0], [0, 0, -10, 0, 3, 0, -13, 0], [-18, 0, 0, 0, 0, -4, 0, 0], [0, -2, -5, 0, 0, 0, 5, 0], [0, 0, 0, 9, 0, 12, 0, -15], [22, 0, 0, 0, 0, -3, 0, 0], [0, 0, 0, -17, 0, -7, 0, -3], [0, 0, 0, 5, 0, 2, 0, 1], [0, 0, -3, 0, -7, 0, 4, 0], [6, 0, 0, 0, 0, 11, 0, 0], [0, 17, 19, 0, 0, 0, -19, 0], [0, 0, 0, -11, 0, -3, 0, -5], [-10, 0, 0, 0, 0, -10, 0, 0], [0, -18, -2, 0, 0, 0, 2, 0], [-14, 0, 0, 0, 0, -4, 0, 0], [0, -1, -5, 0, 0, 0, 5, 0], [0, -8, 13, 0, 0, 0, -13, 0], [0, 0, 0, -2, 0, -12, 0, 22], [0, 0, -2, 0, 3, 0, -5, 0], [14, 0, 0, 0, 0, 20, 0, 0], [6, 0, 0, 0, 0, -20, 0, 0], [0, 11, 2, 0, 0, 0, -2, 0], [0, 6, 5, 0, 0, 0, -5, 0], [0, 0, 16, 0, 8, 0, 8, 0], [-6, 0, 0, 0, 0, 1, 0, 0], [0, -21, -8, 0, 0, 0, 8, 0], [0, 0, 0, 0, 0, 2, 0, -4], [0, 0, 3, 0, 3, 0, 0, 0], [-50, 0, 0, 0, 0, -6, 0, 0], [0, 0, 0, 4, 0, -3, 0, 10], [0, -7, -3, 0, 0, 0, 3, 0], [0, 0, 0, -10, 0, -6, 0, 2], [0, 19, -8, 0, 0, 0, 8, 0], [0, 16, 11, 0, 0, 0, -11, 0], [0, 0, 0, -10, 0, -3, 0, -4], [0, 0, 0, 5, 0, -6, 0, 17], [0, 0, 4, 0, -4, 0, 8, 0], [0, -23, 14, 0, 0, 0, -14, 0], [0, -8, 18, 0, 0, 0, -18, 0], [0, 0, 0, -13, 0, -1, 0, -11], [0, 0, 8, 0, 11, 0, -3, 0], [0, 3, -15, 0, 0, 0, 15, 0], [0, 0, 0, 10, 0, -1, 0, 12], [0, 0, -11, 0, 4, 0, -15, 0], [0, 0, -1, 0, 1, 0, -2, 0], [-34, 0, 0, 0, 0, -7, 0, 0], [14, 0, 0, 0, 0, -4, 0, 0], [0, 14, -14, 0, 0, 0, 14, 0], [0, 0, 2, 0, 2, 0, 0, 0], [0, 0, -7, 0, 0, 0, 7, 0], [0, 0, 18, 0, 5, 0, 13, 0], [0, -5, -18, 0, 0, 0, 18, 0], [0, 0, 0, -2, 0, -2, 0, 2], [0, 0, 0, -13, 0, -6, 0, -1], [0, 0, 4, 0, -8, 0, 12, 0], [0, -5, 8, 0, 0, 0, -8, 0], [0, -16, 14, 0, 0, 0, -14, 0], [0, 0, 16, 0, 3, 0, 13, 0], [0, 0, -9, 0, -7, 0, -2, 0], [0, 0, 0, 5, 0, -1, 0, 7], [0, 0, 0, -12, 0, -2, 0, -8], [0, 0, -9, 0, 1, 0, -10, 0], [22, 0, 0, 0, 0, 1, 0, 0], [58, 0, 0, 0, 0, -2, 0, 0], [0, 8, 12, 0, 0, 0, -12, 0], [0, 0, 0, 4, 0, 0, 0, 4], [0, 0, 5, 0, 16, 0, -11, 0], [50, 0, 0, 0, 0, -4, 0, 0], [2, 0, 0, 0, 0, 10, 0, 0], [0, 25, -16, 0, 0, 0, 16, 0], [0, 10, 13, 0, 0, 0, -13, 0], [0, 0, 0, 6, 0, 12, 0, -18], [0, 0, 0, 5, 0, 5, 0, -5], [0, 0, 0, 11, 0, -4, 0, 19], [0, 0, -8, 0, 9, 0, -17, 0], [-2, 0, 0, 0, 0, 11, 0, 0], [0, 0, 0, 8, 0, 12, 0, -16], [0, 0, 5, 0, -2, 0, 7, 0], [26, 0, 0, 0, 0, -14, 0, 0], [0, 0, 0, 4, 0, -4, 0, 12], [0, 0, -5, 0, 2, 0, -7, 0], [30, 0, 0, 0, 0, -2, 0, 0], [0, 7, -5, 0, 0, 0, 5, 0], [42, 0, 0, 0, 0, 2, 0, 0], [0, 27, 6, 0, 0, 0, -6, 0], [0, -18, 7, 0, 0, 0, -7, 0], [0, 0, 0, 6, 0, 2, 0, 2], [-2, 0, 0, 0, 0, 27, 0, 0], [0, -8, 2, 0, 0, 0, -2, 0], [0, 0, -3, 0, -7, 0, 4, 0], [0, -25, 17, 0, 0, 0, -17, 0], [0, 0, 9, 0, 7, 0, 2, 0], [0, 0, 0, 4, 0, -2, 0, 8], [0, 0, 13, 0, 12, 0, 1, 0], [0, 0, 2, 0, 0, 0, 2, 0], [-42, 0, 0, 0, 0, 12, 0, 0], [46, 0, 0, 0, 0, 10, 0, 0], [0, 17, 13, 0, 0, 0, -13, 0], [0, 0, 0, 0, 0, 11, 0, -22], [18, 0, 0, 0, 0, 8, 0, 0], [0, -18, 19, 0, 0, 0, -19, 0], [0, 0, 0, -19, 0, -6, 0, -7], [0, -4, -22, 0, 0, 0, 22, 0], [0, 0, 0, 17, 0, -1, 0, 19], [-58, 0, 0, 0, 0, 8, 0, 0], [0, -11, 11, 0, 0, 0, -11, 0], [-30, 0, 0, 0, 0, -2, 0, 0], [0, -6, -6, 0, 0, 0, 6, 0], [0, 0, 9, 0, 4, 0, 5, 0], [50, 0, 0, 0, 0, -6, 0, 0], [-6, 0, 0, 0, 0, -10, 0, 0], [0, -31, 13, 0, 0, 0, -13, 0]]
